import React, { useState } from 'react';
import ReactDOM from 'react-dom';
import { scaleTime, scaleLog, extent } from 'd3';

import { data } from './data';
import { AxisBottom } from './components/AxisBottom';
import { AxisLeft } from './components/AxisLeft';
import { Line } from './components/Line';
import { Tooltip } from './components/Tooltip';

const width = 960;
const height = 500;
const margin = { top: 85, right: 60, bottom: 60, left: 90 };
const innerWidth = width - margin.right - margin.left;
const innerHeight = height - margin.top - margin.bottom;

data.forEach((d) => (d.date = new Date(d.date)));

const xScale = scaleTime()
  .domain(extent(data, (d) => d.date))
  .range([0, innerWidth]);

const yScale = scaleLog()
  .domain(extent(data, (d) => d.rank))
  .range([0, innerHeight]);

const App = () => {
  const [hoveredPoint, setHoveredPoint] = useState(data[0]);

  const handleMouseMove = ({ clientX }) => {
    const exactDate = xScale.invert(clientX - margin.left);
    const diffs = data.map((d) =>
      Math.abs(d.date - exactDate)
    );
    const snapIndex = diffs.indexOf(Math.min(...diffs));

    setHoveredPoint(data[snapIndex]);
  };

  return (
    <>
      <h1
        style={{
          marginTop: margin.top / 7,
          marginLeft: margin.left - 3,
        }}
      >
        My Rank on{' '}
        <span className="accent">HackerRank </span>
        <small>JavaScript - Problem Solving</small>
      </h1>
      <svg
        width={width}
        height={height}
        onMouseMove={handleMouseMove}
      >
        <g
          transform={`translate(${margin.left},${margin.top})`}
        >
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            axisOffset={25}
          />
          <AxisLeft
            yScale={yScale}
            innerWidth={innerWidth}
            axisOffset={20}
          />
          <Tooltip
            innerWidth={innerWidth}
            innerHeight={innerHeight}
            hoveredPoint={hoveredPoint}
            xScale={xScale}
          />
          <Line
            data={data}
            xScale={xScale}
            yScale={yScale}
          />
        </g>
      </svg>
    </>
  );
};

ReactDOM.render(<App />, document.getElementById('root'));
